#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Name: Tattletale (tattletale.pl)
# Auth: dumpsterKEEPER
# Date: June 20, 2004
# Desc: Strips chat strings out of SoF2 log files; formerly chat strip,
#       but recoded with additional functionality and form
# -----------------------------------------------------------------------------
use strict;
use Getopt::Std;

# -----------------------------------------------------------------------------
# PROGRAM CONSTANTS
# -----------------------------------------------------------------------------
my $global_text = "To all:";
my $team_text   = "To the team:";
my $admin_text  = "Thus spake the admin:";
my $strip_text  = "tidied up some loose weapons";
my $phone_text  = "made a long distance phone call";
my $strike_text = "fried the brainpan of";
my $bbq_text    = "toasted the buns of";
my $no_text     = "Sorry, no chat lines to display";
my $page_title  = "Chat from The Dumpster [sof2.wbc.edu]";
my $css_path    = "main.css";

# -----------------------------------------------------------------------------
# PROGRAM VARIABLES
# -----------------------------------------------------------------------------
my %opts;        # Command line arguments
my @input_files; # Input log files to parse
my $html_file;   # Path to output HTML file
my $text_file;   # Path to output text file
my $display_num; # Number of chat lines to display
my $viewing_num; # Number of chat lines output
my @chat;        # All chat lines from input files
my $start_time;  # Start for timing code
my $end_time;    # End for timing code

# Flags
my $retain_codes;    # Keep/remove color codes
my $retain_tokens;   # Keep/remove OSP tokens
my $display_actions; # Show/hide admin actions

# Counters
my $num_lines      = 0; # Number of lines scanned
my $num_chat_lines = 0; # Number of chat lines parsed
my $num_global     = 0; # Number of global chat lines
my $num_team       = 0; # Number of team chat lines
my $num_admin      = 0; # Number of admin talk lines
my $num_action     = 0; # Number of admin action lines parsed

# -----------------------------------------------------------------------------
# Main Program
# -----------------------------------------------------------------------------
&get_arguments();
&read_data();
&write_files();

# -----------------------------------------------------------------------------
# Read log file and parse chat lines into array
# -----------------------------------------------------------------------------
sub read_data {
   my $i = 0;
   my $time;
   my $curr_file;
   my $bbq_admin;

   $start_time = (times)[0];
   foreach $curr_file (@input_files) {
      unless(open(LOG, "$curr_file")) {
         print "\nError: Cannot open $curr_file - $!\n";
         next;
      }
      print "Reading $curr_file...\n";
      while (<LOG>) {
         chomp;
         $num_lines++;
         # Example chat lines:
         # 30:16 say: dumpsterKEEPER: diff map?
         # 48:07 sayteam: |S2| SOUTHPORT PSYCHO: he is at extaraction 2
         # 17062:12admin chat: Jiff: i dont like this
         # 25742:14admin talk: MuffinHunter: no cursing RAS
         # 25744:15tell: dumpsterKEEPER to MuffinHunter: you need that headset, man
         if (m/(\d*:\d\d)\s*?say: (.*?): (.*)/) { # Global chat
            ($chat[$i][0], $chat[$i][1], $chat[$i][2], $chat[$i][3], $chat[$i][4]) = ("CG", $1, $2, "", $3);
            $i++;
            $num_global++;
            $num_chat_lines++;
            next;
         } elsif (m/(\d*:\d\d)\s*?(sayteam|sayteamnl): (.*?)\: (.*)/) { # Team chat
            ($chat[$i][0], $chat[$i][1], $chat[$i][2], $chat[$i][3], $chat[$i][4]) = ("CT", $1, $3, "", $4);
            $i++;
            $num_team++;
            $num_chat_lines++;
            next;
         } elsif (m/(\d*:\d\d)\s*?admin talk: (.*?)\: (.*)/) { # Admin talk
            ($chat[$i][0], $chat[$i][1], $chat[$i][2], $chat[$i][3], $chat[$i][4]) = ("CH", $1, $2, "", $3);
            $i++;
            $num_admin++;
            $num_chat_lines++;
            next;
         }
         # Example admin actions:
         # 25735:32Kill: 4 5 27: MuffinHunter killed RF[S by MOD_TARGET_LASER at hit location:0 onsameteam:1
         #   25735:32admin strike: MuffinHunter
         # 17075:54admin bbq: Jiff
         #   17076:16Kill: 1022 5 19: <world> killed K/.Mr.bb by MOD_ANM14_GRENADE at hit location:0 onsameteam:0
         # 17043:10admin strip: Jiff
         # 17083:39admin phone: Jiff
         if ($display_actions) {
            if (m/(\d*:\d\d)\s*?Kill:[\s\d]*?: (.*?) killed (.*) by MOD_TARGET_LASER/) { # Admin strike
               ($chat[$i][0], $chat[$i][1], $chat[$i][2], $chat[$i][3], $chat[$i][4]) = ("AS", $1, $2, $3, "");
               $i++;
               $num_action++;
               next;
            } elsif (m/[\d:]*s*?admin bbq\: (.*) /) { # Admin BBQ (get admin name)
               $bbq_admin = $1;
            } elsif ($bbq_admin && m/(\d*:\d\d)\s*?Kill:[\s\d]*?: <world> killed (.*) by MOD_ANM14_GRENADE/) { # Admin BBQ (get rest of data)
               ($chat[$i][0], $chat[$i][1], $chat[$i][2], $chat[$i][3], $chat[$i][4]) = ("AB", $1, $bbq_admin, $2, "");
               $i++;
               $num_action++;
               $bbq_admin = "";
               next;
            } elsif (m/(\d*:\d\d)\s*?admin strip: (.*) /) { # Admin strip
               ($chat[$i][0], $chat[$i][1], $chat[$i][2], $chat[$i][3], $chat[$i][4]) = ("AT", $1, $2, "", "");
               $i++;
               $num_action++;
               next;
            } elsif (m/(\d*:\d\d)\s*?admin phone: (.*) /) { # Admin phone
               ($chat[$i][0], $chat[$i][1], $chat[$i][2], $chat[$i][3], $chat[$i][4]) = ("AP", $1, $2, "", "");
               $i++;
               $num_action++;
               next;
            }
         }
      }
      close(LOG);
   }
   $end_time = (times)[0];
   print "Scanned $num_lines lines\n";
   print "Parsed $num_chat_lines lines of chat\n";
   print "Parsed $num_action admin actions\n";
}

# -----------------------------------------------------------------------------
# Write chat lines into appropriate format
# -----------------------------------------------------------------------------
sub write_files {
   if ($display_num) {
      $viewing_num = $display_num;
   } else {
      $viewing_num = $num_chat_lines;
   }

   if ($html_file) {
      print "Writing HTML to $html_file\n";
      &write_html_file($html_file);
   }

   if ($text_file) {
      print "Writing text to $text_file\n";
      &write_text_file($text_file);
   }
}

# -----------------------------------------------------------------------------
# Writes chat lines into HTML format
# -----------------------------------------------------------------------------
sub write_html_file {
   my $html_file = shift;
   my $line;
   my $i = 0;

   open(OUT, ">$html_file") || die("\nError: Cannot open $html_file - $!\n");
   &print_html_header(*OUT);
   if ($num_chat_lines) {
      foreach $line (reverse @chat) {
         $i++;
         last if ($i >= $viewing_num);
         switch: {
            if (@$line[0] eq "CG") {
               print OUT format_html("@$line[1]| > $global_text (@$line[2]) @$line[4]") . "<br />\n";
               last switch;
            }
            if (@$line[0] eq "CT") {
               print OUT format_html("@$line[1]| > $team_text (@$line[2]) @$line[4]") . "<br />\n";
               last switch;
            }
            if (@$line[0] eq "CH") {
               print OUT format_html("@$line[1]| > $admin_text (@$line[2]) @$line[4]") . "<br />\n";
               last switch;
            }
            if (@$line[0] eq "AP") {
               print OUT format_html("@$line[1]| > (@$line[2]) $phone_text") . "<br />\n";
               last switch;
            }
            if (@$line[0] eq "AS") {
               print OUT format_html("@$line[1]| > (@$line[2]) $strike_text @$line[3]") . "<br />\n";
               last switch;
            }
            if (@$line[0] eq "AB") {
               print OUT format_html("@$line[1]| > (@$line[2]) $bbq_text @$line[3]") . "<br />\n";
               last switch;
            }
            if (@$line[0] eq "AT") {
               print OUT format_html("@$line[1]| > (@$line[2]) $strip_text") . "<br />\n";
               last switch;
            }
         }
      }
   } else {
      print OUT "| > $no_text<br />\n";
   }
   &print_html_footer(*OUT);
   close(OUT);
}

# -----------------------------------------------------------------------------
# Writes chat lines into text format
# -----------------------------------------------------------------------------
sub write_text_file {
   my $text_file = shift;
   my $line;
   my $i = 0;

   open(OUT, ">$text_file") || die("\nError: Cannot open $text_file - $!\n");
   &print_text_header(*OUT);
   if ($num_chat_lines) {
      foreach $line (@chat) {
         $i++;
         last if ($i >= $viewing_num);
         print OUT "//" . format_text(@$line[1]) . "//" . format_text(@$line[2]);
         print OUT "//" . format_text(@$line[3]) . "//" . format_text(@$line[4]) . "//\n";
      }
   } else {
      print OUT "$no_text\n";
   }
   close(OUT);
}


# -----------------------------------------------------------------------------
# Format the text to remove HTML specific characters
# -----------------------------------------------------------------------------
sub format_html {
   my $html = shift;

   &detokenize(\$html);

   # Encode nasty ASCII characters to HTML
   $html =~ s/</\&lt\;/g;   # Left bracket
   $html =~ s/>/\&gt\;/g;   # Right bracket
   $html =~ s/ /\&nbsp\;/g; # Space
   $html =~ s/\n/<br \/>/g; # Newline

   return $html;
}

# -----------------------------------------------------------------------------
# Format the text to remove illegal/unwanted characters
# -----------------------------------------------------------------------------
sub format_text {
   my $text = shift;

   &detokenize(\$text);
   $text =~ s/\/\//_/g;

   return $text;
}

# -----------------------------------------------------------------------------
# Remove color codes and OSP tokens
# -----------------------------------------------------------------------------
sub detokenize {
   my $text = shift;

   # Remove color codes (i.e. ^1, ^2, #44;, etc)
   unless ($retain_codes) {
      $$text =~ s/\^.+?//g;
      $$text =~ s/#\d\d\;//g;
   }

   # Replace OSP chat tokens
   unless ($retain_tokens) {
      $$text =~ s/#a/%armor%/gi;     # Current armor
      $$text =~ s/#b/%armor bar%/g;  # Current armor (bar format)
      $$text =~ s/#h/%health%/gi;    # Current health
      $$text =~ s/#B/%health bar%/g; # Current health (bar format)
      $$text =~ s/#d/%enemy%/gi;     # Last player to hurt you
      $$text =~ s/#t/%target%/gi;    # Last player you hurt
      $$text =~ s/#f/%teammate%/gi;  # Closest teammate
      $$text =~ s/#l/%location%/gi;  # Location in level
      $$text =~ s/#i/%pistol%/gi;    # Current pistol
      $$text =~ s/#p/%primary%/gi;   # Current primary weapon
      $$text =~ s/#s/%secondary%/gi; # Current secondary weapon
      $$text =~ s/#g/%grenades%/gi;  # Current grenades
      $$text =~ s/#e/%equipment%/gi; # Current equipment
   }
}

# -----------------------------------------------------------------------------
# Print header for output HTML file
# -----------------------------------------------------------------------------
sub print_html_header {
   local *OUT = shift;
   my ($percent_global, $percent_team, $percent_admin);
   my $curr_date = localtime;

   # Calculate chat percentages
   if ($num_chat_lines) {
      $percent_global = sprintf("%.0f", ($num_global / $num_chat_lines) * 100);
      $percent_team   = sprintf("%.0f", ($num_team / $num_chat_lines) * 100);
      $percent_admin  = sprintf("%.0f", ($num_admin / $num_chat_lines) * 100);
   } else {
      $percent_global = 0;
      $percent_team   = 0;
      $percent_admin  = 0;
   }

   print OUT <<HEADER;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
   <title>$page_title</title>
   <meta http-equiv="content-type" content="text/html; charset=iso-8859-1" />
   <link rel="stylesheet" type="text/css" href="$css_path" />
</head><body>
<div id="Header">
   | > <br />
   | > <strong>Tattletale</strong><br />
   | > <br />
   | > Topic: ($page_title)<br />
   | > Synced: ($curr_date)<br />
   | > <br />
   | > Processed: ($num_lines) lines<br />
   | > Chat: ($num_chat_lines), Global: ($num_global/$percent_global%), Team: ($num_team/$percent_team%), Admin: ($num_admin/$percent_admin%)<br />
   | > Actions: ($num_action)<br />
   | > Viewing: ($viewing_num) lines<br />
   | > <br />
</div>
<div id="Body">
   | > <br />
HEADER
}

# -----------------------------------------------------------------------------
# Print footer for output HTML file
# -----------------------------------------------------------------------------
sub print_html_footer {
   local *OUT = shift;
   my $total_time = sprintf("%.4f", $end_time - $start_time);

   print OUT <<FOOTER;
   | > <br />
</div>
<div id="Footer">
   | > <br />
   | > Created: ($total_time) secs<br />
   | > Bye<br />
   | > <br />
</div>
</body></html>
FOOTER
}

# -----------------------------------------------------------------------------
# Print header for output text file
# -----------------------------------------------------------------------------
sub print_text_header {
   local *OUT = shift;
   my ($percent_global, $percent_team, $percent_admin);
   my $curr_date = localtime;

   # Calculate chat percentages
   if ($num_chat_lines) {
      $percent_global = sprintf("%.0f", ($num_global / $num_chat_lines) * 100);
      $percent_team   = sprintf("%.0f", ($num_team / $num_chat_lines) * 100);
      $percent_admin  = sprintf("%.0f", ($num_admin / $num_chat_lines) * 100);
   } else {
      $percent_global = 0;
      $percent_team   = 0;
      $percent_admin  = 0;
   }

   print OUT <<HEADER;
# Tattletale ($page_title) initiated $curr_date
# Processed: $num_lines lines, Viewing: $viewing_num lines
# Chat: $num_chat_lines, Global: $num_global/$percent_global%, Team: $num_team/$percent_team%, Admin: $num_admin/$percent_admin%
# Actions: $num_action
# Fields are: Log time stamp, initiator, receiver, text
HEADER
}

# -----------------------------------------------------------------------------
# Retrieve and process command line arguments
# -----------------------------------------------------------------------------
sub get_arguments {
   getopts('o:x:n:csath', \%opts) or &print_usage();

   # Print help/usage information to the screen if necessary
   &print_usage() if ($opts{h});
   &print_usage() unless ($ARGV[0]);
   &print_usage() unless ($opts{o} || $opts{x});
   if ($opts{n}) {
      &print_usage() unless ($opts{n} =~ /\d/); # Ensure numeric
   }

   # Copy command line arguments to internal variables
   $html_file       = $opts{o} if ($opts{o});
   $text_file       = $opts{x} if ($opts{x});
   $display_num     = 0 unless ($display_num     = $opts{n});
   $retain_codes    = 0 unless ($retain_codes    = $opts{c});
   $retain_tokens   = 0 unless ($retain_tokens   = $opts{s});
   $display_actions = 0 unless ($display_actions = $opts{a});
   @input_files     = @ARGV;
}

# -----------------------------------------------------------------------------
# Print usage/help information to the screen and exit
# -----------------------------------------------------------------------------
sub print_usage {
   die <<USAGE;
 +--------------------------------------------------------------+
 |              Tattletale -- SoF2 Log Chat Parser              |
 +--------------------------------------------------------------+

 Usage:
    perl tattletale.pl [ -o html file ] [ -x text file ]
                       [-n display count] [ -csa ]
                       [ file1 .. fileN ]

 -o | Output filename for HTML output
 -x | Output filename for text output
 -n | Limit the number of chat lines displayed
 -c | Retain color codes (^3)
 -s | Retain OSP tokens (#T)
 -a | Display admin actions: strike, bbq, strip, and phone.

 Additional Options:
    -h | Prints this help/usage information to the screen

 Examples:
    perl tattletale.pl -o out.html -n 300 -a in.log
    perl tattletale.pl -x out.txt -c -s in1.log in2.log
USAGE
}
